@@ -4,13 +4,18 @@ use strict;
 use warnings;
 use Carp;
 use Storable;
+use File::Temp ();
+use File::Path qw[rmtree];
+use File::Spec;
+use POSIX qw( O_CREAT O_RDWR O_RDONLY );         # for SDBM_File
+use SDBM_File;
 use POE qw[Wheel::Run Filter::Line];
 use Digest::SHA qw[sha256];
 use Env::Sanctify;
 use Module::Pluggable search_path => 'POE::Component::SmokeBox::Backend', sub_name => 'backends', except => 'POE::Component::SmokeBox::Backend::Base';
 use vars qw($VERSION);
 
-$VERSION = '0.42';
+$VERSION = '0.44';
 
 my $GOT_KILLFAM;
 my $GOT_PTY;
@@ -167,7 +172,9 @@ sub _start {
   $kernel->detach_myself() if $kernel != $sender;
 
   $self->{_wheel_log} = [ ];
-  $self->{_digests} = { };
+
+  $self->_tie_digests();
+
   $self->{_loop_detect} = 0;
   $self->{start_time} = time();
 
@@ -177,11 +184,35 @@ sub _start {
 
 sub _shutdown {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
+  $self->_untie_digests();
   $self->{term_kill} = 1;
   $kernel->yield( '_wheel_kill', 'Killing current due to component shutdown event' );
   return;
 }
 
+# Digests tie and untie
+
+sub _tie_digests {
+  my $self = shift;
+  $self->{_tempdir} = File::Temp->newdir();
+  $self->{_tmpdirname} = $self->{_tempdir}->dirname;
+  my $file = File::Spec->catfile( $self->{_tmpdirname}, 'digests.dat' );
+  $self->{_digests} = { };
+  tie %{ $self->{_digests} }, 'SDBM_File', $file, O_CREAT|O_RDWR, 0644 or die "Could not tie: $!\n";
+  return 1;
+}
+
+sub _untie_digests {
+  my $self = shift;
+  if ( $self->{_digests} ) {
+    untie %{ $self->{_digests} };
+    delete $self->{_digests};
+    delete $self->{_tempdir};
+    rmtree( $self->{_tmpdirname} ) if -d $self->{_tmpdirname};
+  }
+  return 1;
+}
+
 sub _spawn_wheel {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
 
@@ -252,7 +283,9 @@ sub _finalize_job {
   my( $self, $status ) = @_;
 
   $self->{end_time} = time();
-  delete $self->{_digests};
+
+  $self->_untie_digests();
+
   delete $self->{_loop_detect};
 
   my $job = { };
@@ -323,7 +356,12 @@ sub _detect_loop {
     $weighting = $handle eq 'stderr' ? 1 : 10;
   }
 
-  $self->{_digests}->{ $digest } += $weighting;
+  if ( exists $self->{_digests}->{ $digest } ) {
+    $self->{_digests}->{ $digest } += $weighting;
+  }
+  else {
+    $self->{_digests}->{ $digest } = $weighting;
+  }
   return unless ++$self->{_digests}->{ $digest } > 3000;
   return $self->{_loop_detect} = 1;
 }
